@@ -1,9 +1,7 @@
 const St = imports.gi.St;
 const ModalDialog = imports.ui.modalDialog;
 const Gio = imports.gi.Gio;
-const Gtk = imports.gi.Gtk;
 const Lang = imports.lang;
-const Clutter = imports.gi.Clutter;
 const Applet = imports.ui.applet;
 const PopupMenu = imports.ui.popupMenu;
 const Mainloop = imports.mainloop;
@@ -34,7 +32,7 @@ MyApplet.prototype = {
 
             this._onTrashChange();
 
-            this.monitor = this.trash_directory.monitor_directory(0, null, null);
+            this.monitor = this.trash_directory.monitor_directory(0, null);
             this.monitor.connect('changed', Lang.bind(this, this._onTrashChange));
         }
         catch (e) {
@@ -77,7 +75,7 @@ MyApplet.prototype = {
         this.trash_changed_timeout = 0;
         if (this.trash_directory.query_exists(null)) {
             let children = this.trash_directory.enumerate_children('standard::*', Gio.FileQueryInfoFlags.NONE, null);
-            if (children.next_file(null, null) == null) {
+            if (children.next_file(null) == null) {
                 this.set_applet_icon_symbolic_name("user-trash");
             } else {
                 this.set_applet_icon_symbolic_name("user-trash-full");
@@ -95,7 +93,7 @@ MyApplet.prototype = {
     }
 };
 
-function main(metadata, orientation, panel_height, instance_id) {      
+function main(metadata, orientation, panel_height, instance_id) {
     let myApplet = new MyApplet(orientation, panel_height, instance_id);
-    return myApplet;      
+    return myApplet;
 }
