@@ -26,6 +26,7 @@ DEF_HEADING = 2
 preferred_app_defs = [
     # 1st mimetype is to let us find apps
     # 2nd mimetype is to set default handler for (so we handle all of that type, not just a specific format)
+    ( "inode/directory",         "inode/directory",            _("File Manager") ),
     ( "x-scheme-handler/http",   "x-scheme-handler/http",      _("_Web") ),
     ( "x-scheme-handler/mailto", "x-scheme-handler/mailto",    _("_Mail") ),
     ( "application/msword",      "application/msword",       _("Documents") ),
@@ -38,78 +39,78 @@ preferred_app_defs = [
 
 mimetypes = {}
 mimetypes["audio"]=[
-  "audio/3gpp",
-  "audio/aac",
-  "audio/ac3",
-  "audio/flac",
-  "audio/m4a",
-  "audio/midi",
-  "audio/mp3",
-  "audio/mp4",
-  "audio/mp4a-latm",
-  "audio/mpeg",
-  "audio/mpeg3",
-  "audio/mpg",
-  "audio/ogg",
-  "audio/vorbis",
-  "audio/wav",
-  "audio/wave",
-  "audio/webm",
-  "audio/x-aac",
-  "audio/x-aiff",
-  "audio/x-flac",
-  "audio/x-mp3",
-  "audio/x-mpeg",
-  "audio/x-mpeg-3",
-  "audio/x-mpg",
-  "audio/x-ms-asf",
-  "audio/x-ms-wma",
-  "audio/x-ogg",
-  "audio/x-oggflac",
-  "audio/x-vorbis",
-  "audio/x-vorbis+ogg",
-  "audio/x-wav",
-  "audio/x-wavpack"
+    "audio/3gpp",
+    "audio/aac",
+    "audio/ac3",
+    "audio/flac",
+    "audio/m4a",
+    "audio/midi",
+    "audio/mp3",
+    "audio/mp4",
+    "audio/mp4a-latm",
+    "audio/mpeg",
+    "audio/mpeg3",
+    "audio/mpg",
+    "audio/ogg",
+    "audio/vorbis",
+    "audio/wav",
+    "audio/wave",
+    "audio/webm",
+    "audio/x-aac",
+    "audio/x-aiff",
+    "audio/x-flac",
+    "audio/x-mp3",
+    "audio/x-mpeg",
+    "audio/x-mpeg-3",
+    "audio/x-mpg",
+    "audio/x-ms-asf",
+    "audio/x-ms-wma",
+    "audio/x-ogg",
+    "audio/x-oggflac",
+    "audio/x-vorbis",
+    "audio/x-vorbis+ogg",
+    "audio/x-wav",
+    "audio/x-wavpack"
 ]
 
 mimetypes["video"]=[
-  "video/3gp",
-  "video/3gpp",
-  "video/divx",
-  "video/flv",
-  "video/mp4",
-  "video/mp4v-es",
-  "video/mpeg",
-  "video/msvideo",
-  "video/ogg",
-  "video/quicktime",
-  "video/vivo",
-  "video/vnd.divx",
-  "video/vnd.rn-realvideo",
-  "video/webm",
-  "video/x-anim",
-  "video/x-avi",
-  "video/x-flc",
-  "video/x-fli",
-  "video/x-flic",
-  "video/x-flv",
-  "video/x-m4v",
-  "video/x-matroska",
-  "video/x-mng",
-  "video/x-mpeg",
-  "video/x-mpeg2",
-  "video/x-ms-afs",
-  "video/x-ms-asf",
-  "video/x-ms-asx",
-  "video/x-ms-wm",
-  "video/x-ms-wmv",
-  "video/x-ms-wvx",
-  "video/x-ms-wvxvideo",
-  "video/x-msvideo",
-  "video/x-nsv",
-  "video/x-ogm+ogg",
-  "video/x-theora",
-  "video/x-theora+ogg"
+    "video/3gp",
+    "video/3gpp",
+    "video/divx",
+    "video/flv",
+    "video/mp4",
+    "video/mp4v-es",
+    "video/mpeg",
+    "video/msvideo",
+    "video/ogg",
+    "video/quicktime",
+    "video/vivo",
+    "video/vnd.divx",
+    "video/vnd.rn-realvideo",
+    "video/webm",
+    "video/x-anim",
+    "video/x-avi",
+    "video/x-flc",
+    "video/x-fli",
+    "video/x-flic",
+    "video/x-flv",
+    "video/x-m4v",
+    "video/x-matroska",
+    "video/x-mng",
+    "video/x-mpeg",
+    "video/x-mpeg2",
+    "video/x-ms-afs",
+    "video/x-ms-asf",
+    "video/x-ms-asx",
+    "video/x-ms-wm",
+    "video/x-ms-wmv",
+    "video/x-ms-wvx",
+    "video/x-ms-wvxvideo",
+    "video/x-msvideo",
+    "video/x-nsv",
+    "video/x-ogm+ogg",
+    "video/x-theora",
+    "video/x-theora+ogg"
 ]
 
 mimetypes["text/x-python"] = [
@@ -192,6 +193,8 @@ class MnemonicLabel(Gtk.Label):
         super(MnemonicLabel, self).__init__(label = "")
         self.set_text_with_mnemonic(text)
         self.set_mnemonic_widget(widget)
+        self.set_alignment(0.0, 0.5)
+        self.set_line_wrap(True)
 
 class DefaultAppChooserButton(Gtk.AppChooserButton):
     def __init__(self, content_type, gen_content_type):
@@ -276,6 +279,8 @@ class CustomAppChooserButton(Gtk.AppChooserButton):
         self.media_settings = media_settings
         content_type = self.get_content_type()
 
+        self.set_valign(Gtk.Align.CENTER)
+
         #fetch preferences for this content type
         (pref_start_app, pref_ignore, pref_open_folder) = self.getPreferences()
         pref_ask = not pref_start_app and not pref_ignore and not pref_open_folder
@@ -348,9 +353,10 @@ class CustomAppChooserButton(Gtk.AppChooserButton):
         self.setPreference(pref_open_folder, PREF_MEDIA_AUTORUN_X_CONTENT_OPEN_FOLDER)
 
 class OtherTypeDialog(Gtk.Dialog):
-    def __init__(self, media_settings):
+    def __init__(self, media_settings, transient_parent):
         super(OtherTypeDialog, self).__init__(title = _("Other Media"),
-                                              transient_for = None,
+                                              transient_for = transient_parent,
+                                              border_width = 6,
                                               flags = 0)
         self.add_button(_("Close"), Gtk.ResponseType.OK)
 
@@ -467,7 +473,7 @@ class Module:
             print "Loading Default module"
 
             self.media_settings = Gio.Settings.new(MEDIA_HANDLING_SCHEMA)
-            self.other_type_dialog = OtherTypeDialog(self.media_settings)
+            self.other_type_dialog = OtherTypeDialog(self.media_settings, self.sidePage.window)
 
             self.sidePage.stack = SettingsStack()
             self.sidePage.add_widget(self.sidePage.stack)
@@ -526,12 +532,16 @@ class Module:
                 widget.pack_end(button, False, False, 0)
                 settings.add_row(widget)
 
-            button = Button(_("_Other Media..."), self.onMoreClicked)
+            # FIXMEEEEEEEE??
+            button = Button(_("_Other Media...").strip("_"), self.onMoreClicked)
             settings.add_row(button)
 
     def onMoreClicked(self, widget):
         self.other_type_dialog.doShow(widget.get_toplevel())
 
+    def _setParentRef(self, window):
+        self.sidePage.window = window
+
 class InvertedSwitch(SettingsWidget):
     def __init__(self, label, schema, key):
         self.key = key
