@@ -7,7 +7,7 @@ import os, gettext, sys
 sys.path.append('/usr/lib/linuxmint/common')
 import additionalfiles
 
-os.environ['LANG'] = "en_US.UTF-8"
+os.environ['LANGUAGE'] = "en_US.UTF-8"
 gettext.install(DOMAIN, PATH)
 
 prefix = """[Desktop Entry]
@@ -35,3 +35,15 @@ Keywords=Preferences;Settings;
 """
 
 additionalfiles.generate(DOMAIN, PATH, "files/usr/share/applications/cinnamon-settings-users.desktop", prefix, _("Users and Groups"), _("Add or remove users and groups"), "")
+
+prefix = """[Desktop Entry]
+Exec=dbus-send --print-reply --dest=org.Cinnamon /org/Cinnamon org.Cinnamon.ToggleKeyboard
+Icon=keyboard
+Terminal=false
+Type=Application
+Categories=Utility;
+OnlyShowIn=X-Cinnamon;
+Keywords=onboard;keyboard;caribou;
+"""
+
+additionalfiles.generate(DOMAIN, PATH, "files/usr/share/applications/cinnamon-onscreen-keyboard.desktop", prefix, _("Virtual keyboard"), _("Turn on-screen keyboard on or off"), "")
