@@ -112,6 +112,20 @@ SignalManager.prototype = {
         this._storage.push([sigName, obj, callback, id]);
     },
 
+    connect_after: function(obj, sigName, callback, bind, force) {
+        if (!force && this.isConnected(sigName, obj, callback))
+            return
+
+        let id;
+
+        if (bind)
+            id = obj.connect_after(sigName, Lang.bind(bind, callback));
+        else
+            id = obj.connect_after(sigName, Lang.bind(this._object, callback));
+
+        this._storage.push([sigName, obj, callback, id]);
+    },
+
     _signalIsConnected: function (signal) {
         if (!signal[1])
             return false;
@@ -145,7 +159,7 @@ SignalManager.prototype = {
      * Returns: Whether the signal is connected
      */
     isConnected: function() {
-        return (this.getSignals(arguments).length > 0);
+        return (this.getSignals.apply(this, arguments).length > 0);
     },
 
     /**
@@ -192,10 +206,10 @@ SignalManager.prototype = {
      * checks need not be performed before calling this function.
      */
     disconnect: function() {
-        let results = this.getSignals(arguments);
+        let results = this.getSignals.apply(this, arguments);
         results.filter(this._signalIsConnected).forEach(x => x[1].disconnect(x[3]));
 
-        this._storage = this._storage.filter(x => results.indexOf(x) != -1);
+        this._storage = this._storage.filter(x => results.indexOf(x) == -1);
     },
 
     /**
