@@ -11,6 +11,7 @@ const AppletManager = imports.ui.appletManager;
 const DeskletManager = imports.ui.deskletManager;
 const ExtensionSystem = imports.ui.extensionSystem;
 const SearchProviderManager = imports.ui.searchProviderManager;
+const ModalDialog = imports.ui.modalDialog;
 const Util = imports.misc.util;
 const Cinnamon = imports.gi.Cinnamon;
 
@@ -100,8 +101,25 @@ const CinnamonIface =
             <method name="PushSubprocessResult"> \
                 <arg type="i" direction="in" name="process_id" /> \
                 <arg type="s" direction="in" name="result" /> \
+                <arg type="b" direction="in" name="success" /> \
             </method> \
             <method name="ToggleKeyboard"/> \
+            <method name="OpenSpicesAbout"> \
+                <arg type="s" direction="in" name="uuid" /> \
+                <arg type="s" direction="in" name="type" /> \
+            </method> \
+            <method name="GetMonitors"> \
+                <arg type="ai" direction="out" name="monitors" /> \
+            </method> \
+            <method name="GetMonitorWorkRect"> \
+                <arg type="i" direction="in" name="monitor" /> \
+                <arg type="ai" direction="out" name="rect" /> \
+            </method> \
+            <signal name="MonitorsChanged"/> \
+            <method name="GetRunState"> \
+               <arg type="i" direction="out" name="state" /> \
+            </method> \
+            <signal name="RunStateChanged"/> \
         </interface> \
     </node>';
 
@@ -113,6 +131,12 @@ CinnamonDBus.prototype = {
     _init: function() {
         this._dbusImpl = Gio.DBusExportedObject.wrapJSObject(CinnamonIface, this);
         this._dbusImpl.export(Gio.DBus.session, '/org/Cinnamon');
+
+        /* Although this signal comes from muffin, it is actually initiated by the
+         * layoutManager.Chrome.updateRegions method.  Workspace code in muffin filters
+         * out chrome updates that don't actually change the workarea before emitting this
+         * signal, which is desirable. */
+        global.screen.connect("workareas-changed", ()=> this.EmitMonitorsChanged());
     },
 
     /**
@@ -213,7 +237,7 @@ CinnamonDBus.prototype = {
             params[param] = params[param].deep_unpack();
 
         let monitorIndex = -1;
-        if (params['monitor'] >= 0) {
+        if (params.maybeGet('monitor') >= 0) {
             monitorIndex = params['monitor'];
         }
 
@@ -306,7 +330,7 @@ CinnamonDBus.prototype = {
 
     highlightXlet: function(uuid, instance_id, highlight) {
         let obj = this._getXletObject(uuid, instance_id);
-        if (obj.highlight) obj.highlight(highlight);
+        if (obj && obj.highlight) obj.highlight(highlight);
     },
 
     highlightPanel: function(id, highlight) {
@@ -368,15 +392,68 @@ CinnamonDBus.prototype = {
             Main.expo.toggle();
     },
 
-    PushSubprocessResult: function(process_id, result) {
+    PushSubprocessResult: function(process_id, result, success) {
         if (Util.subprocess_callbacks[process_id]) {
-            Util.subprocess_callbacks[process_id](result);
+            if (success)
+                Util.subprocess_callbacks[process_id](result);
+            delete Util.subprocess_callbacks[process_id];
         }
     },
 
     ToggleKeyboard: function() {
         Main.keyboard.toggle();
     },
+    
+    OpenSpicesAbout: function(uuid, type) {
+        let metadata = Extension.getMetadata(uuid, Extension.Type[type.toUpperCase()]);
+        new ModalDialog.SpicesAboutDialog(metadata, type+"s");
+    },
+
+    GetMonitors: function() {
+        let monitors = [];
+
+        try {
+            for (let i = 0; i < Main.layoutManager.monitors.length; i++) {
+                let current = Main.layoutManager.monitors[i];
+
+                monitors.push(current.index);
+            }
+        } catch (e) {
+            log(e.message);
+            /* Something broke, trigger a GDBus.Error back to the caller instead of returning bad things */
+            monitors = [];
+        }
+
+        if (monitors.length == 0) {
+            throw new Error("GetMonitors: no valid monitors");
+        }
+
+        return monitors;
+    },
+
+    GetMonitorWorkRect: function(index) {
+        let n_mons = global.screen.get_n_monitors();
+
+        if ((index < 0) || index > (n_mons - 1)) {
+            throw new Error("GetMonitorWorkRect: invalid monitor index: " + index + ".  Must be 0 to " + (n_mons - 1));
+        }
+
+        let rect = global.screen.get_active_workspace().get_work_area_for_monitor(index);
+
+        return [rect.x, rect.y, rect.width, rect.height];
+    },
+
+    GetRunState: function() {
+        return Main.runState;
+    },
+
+    EmitRunStateChanged: function() {
+        this._dbusImpl.emit_signal('RunStateChanged', null);
+    },
+
+    EmitMonitorsChanged: function() {
+        this._dbusImpl.emit_signal('MonitorsChanged', null);
+    },
 
     CinnamonVersion: Config.PACKAGE_VERSION
 };
