@@ -50,13 +50,12 @@ ExpoWindowClone.prototype = {
         this.realWindow = realWindow;
         this.metaWindow = realWindow.meta_window;
         this.refreshClone();
-        this._signalManager = new SignalManager.SignalManager(this);
+        this._signalManager = new SignalManager.SignalManager(null);
 
-        this._signalManager.connect(this.realWindow, 'position-changed', this.onPositionChanged);
-        this._signalManager.connect(this.realWindow, 'size-changed', this.onSizeChanged);
+        this._signalManager.connect(this.realWindow, 'size-changed', this.onSizeChanged, this);
         this._signalManager.connect(this.metaWindow, 'workspace-changed', function(w, oldws) {
             this.emit('workspace-changed', oldws);
-        })
+        }, this);
 
         this.onPositionChanged();
         this.onSizeChanged();
@@ -99,7 +98,7 @@ ExpoWindowClone.prototype = {
         this.inDrag = false;
         this.dragCancelled = false;
 
-        this.icon = new St.Group();
+        this.icon = new St.Widget();
         this.actor.add_actor(this.icon);
         this.icon.hide();
 
@@ -134,7 +133,7 @@ ExpoWindowClone.prototype = {
 
     refreshClone: function(withTransients) {
         if (this.clone) {this.clone.destroy();}
-        this.clone = new St.Group({reactive: false});
+        this.clone = new St.Widget({ reactive: false });
         this.actor.add_actor(this.clone);
         let [pwidth, pheight] = [this.realWindow.width, this.realWindow.height];
         let clones = WindowUtils.createWindowClone(this.metaWindow, 0, 0, withTransients);
@@ -298,11 +297,14 @@ ExpoWorkspaceThumbnail.prototype = {
     _init : function(metaWorkspace, box) {
         this.box = box;
         this.metaWorkspace = metaWorkspace;
-        this.frame = new St.Group({ clip_to_allocation: true,
-                                    style_class: 'expo-workspace-thumbnail-frame' });
-        this.actor = new St.Group({ reactive: true,
-                                    clip_to_allocation: true,
-                                    style_class: 'workspace-thumbnail' });
+
+        this.overviewMode = false;
+
+        this.frame = new St.Widget({ clip_to_allocation: true,
+                                     style_class: 'expo-workspace-thumbnail-frame' });
+        this.actor = new St.Widget({ reactive: true,
+                                     clip_to_allocation: true,
+                                     style_class: 'workspace-thumbnail' });
         this.actor._delegate = this;
         this.actor.set_size(global.screen_width, global.screen_height);
 
@@ -464,9 +466,8 @@ ExpoWorkspaceThumbnail.prototype = {
                 // Use the stable sequence for an integer to use as a hash key
                 this.stackIndices[stack[i].get_meta_window().get_stable_sequence()] = i;
             }
+            this.syncStacking(this.stackIndices);
         }
-
-        this.syncStacking(this.stackIndices);
     },
 
     setActive: function(isActive) {
@@ -959,7 +960,7 @@ ExpoWorkspaceThumbnail.prototype = {
             if (dropping) {
                 metaWindow.move_to_monitor(targetMonitor);
                 if (targetMonitor === Main.layoutManager.primaryIndex) {
-                    metaWindow.change_workspace(this.metaWorkspace, false, time);
+                    metaWindow.change_workspace(this.metaWorkspace);
                 }
             }
         }
@@ -973,7 +974,7 @@ ExpoWorkspaceThumbnail.prototype = {
             {
                 canDrop = true;
                 if (dropping) {
-                    metaWindow.change_workspace(this.metaWorkspace, false, time);
+                    metaWindow.change_workspace(this.metaWorkspace);
                 }
             }
             if (movingMonitors) {
@@ -1659,8 +1660,8 @@ ExpoThumbnailsBox.prototype = {
             x += thumbnailWidth + spacing;
             y += (i + 1) % nColumns > 0 ? 0 : thumbnailHeight + extraHeight + thTitleMargin;
         }
-        let x = 0;
-        let y = 0;
+        x = 0;
+        y = 0;
 
         let buttonWidth = this.button.get_theme_node().get_length('width');
         let buttonHeight = this.button.get_theme_node().get_length('height');
