@@ -14,6 +14,9 @@ const MessageTray = imports.ui.messageTray;
 
 const KEYBOARD_SCHEMA = 'org.cinnamon.keyboard';
 const KEYBOARD_TYPE = 'keyboard-type';
+const KEYBOARD_SIZE = 'keyboard-size';
+const KEYBOARD_POSITION = 'keyboard-position';
+const ACTIVATION_MODE = 'activation-mode';
 
 const A11Y_APPLICATIONS_SCHEMA = 'org.cinnamon.desktop.a11y.applications';
 const SHOW_KEYBOARD = 'screen-keyboard-enabled';
@@ -52,6 +55,7 @@ Key.prototype = {
         this._key = key;
 
         this.actor = this._makeKey();
+        this.actor.connect('destroy', Lang.bind(this, this._onDestroy));
 
         this._extended_keys = this._key.get_extended_keys();
         this._extended_keyboard = null;
@@ -81,6 +85,13 @@ Key.prototype = {
         }
     },
 
+    _onDestroy: function() {
+        if (this._boxPointer) {
+            this._boxPointer.actor.destroy();
+            this._boxPointer = null;
+        }
+    },
+
     _makeKey: function () {
         let label = GLib.markup_escape_text(this._key.label, -1);
         let button = new St.Button ({ label: label,
@@ -208,8 +219,13 @@ Keyboard.prototype = {
 
     _settingsChanged: function (settings, key) {
         this._enableKeyboard = this._a11yApplicationsSettings.get_boolean(SHOW_KEYBOARD);
+        this.accessibleMode = this._keyboardSettings.get_string(ACTIVATION_MODE) == "accessible";
+        this.keyboard_size = this._keyboardSettings.get_int(KEYBOARD_SIZE);
+        this.keyboard_position = this._keyboardSettings.get_string(KEYBOARD_POSITION);
+
         if (!this._enableKeyboard && !this._keyboard)
             return;
+
         if (this._enableKeyboard && this._keyboard &&
             this._keyboard.keyboard_type == this._keyboardSettings.get_string(KEYBOARD_TYPE))
             return;
@@ -368,7 +384,7 @@ Keyboard.prototype = {
         Main.layoutManager.keyboardBox.set_size(focus.width, -1);
         this.actor.width = focus.width;
 
-        let maxHeight = focus.height / 3;
+        let maxHeight = focus.height / this.keyboard_size;
 
         this.monitorIndex = index;
 
@@ -387,8 +403,12 @@ Keyboard.prototype = {
         let keySize = Math.min(keyWidth, keyHeight);
         this.actor.height = (keySize * this._numOfVertKeys) + allVerticalSpacing + (2 * padding) + this._panelPadding;
 
-        Main.layoutManager.keyboardBox.set_position(focus.x,
-                                                    focus.y + focus.height - this.actor.height);
+        let keyboard_y = 0;
+        if (this.keyboard_position == "bottom") {
+            keyboard_y = focus.y + focus.height - this.actor.height;
+        }
+
+        Main.layoutManager.keyboardBox.set_position(focus.x, keyboard_y);
 
         let rows = this._current_page.get_children();
         for (let i = 0; i < rows.length; ++i) {
@@ -479,12 +499,13 @@ Keyboard.prototype = {
     shouldTakeEvent: function(event) {
         let actor = event.get_source();
         return Main.layoutManager.keyboardBox.contains(actor) ||
-               actor._extended_keys || actor.extended_key;
+               actor.maybeGet("_extended_keys") ||
+               actor.maybeGet("extended_key");
     },
 
     // D-Bus methods
     Show: function(timestamp) {
-        if (!this._enableKeyboard)
+        if (!this._enableKeyboard || !this.accessibleMode)
             return;
 
         if (this._compareTimestamp(timestamp, this._timestamp) < 0)
@@ -496,7 +517,7 @@ Keyboard.prototype = {
     },
 
     Hide: function(timestamp) {
-        if (!this._enableKeyboard)
+        if (!this._enableKeyboard || !this.accessibleMode)
             return;
 
         if (this._compareTimestamp(timestamp, this._timestamp) < 0)
@@ -508,17 +529,13 @@ Keyboard.prototype = {
     },
 
     SetCursorLocation: function(x, y, w, h) {
-        if (!this._enableKeyboard)
+        if (!this._enableKeyboard || !this.accessibleMode)
             return;
-
-//        this._setLocation(x, y);
     },
 
     SetEntryLocation: function(x, y, w, h) {
-        if (!this._enableKeyboard)
+        if (!this._enableKeyboard || !this.accessibleMode)
             return;
-
-//        this._setLocation(x, y);
     },
 
     get Name() {
