@@ -66,10 +66,10 @@ ModalDialog.prototype = {
         this._hasModal = false;
         this._cinnamonReactive = params.cinnamonReactive;
 
-        this._group = new St.Group({ visible: false,
-                                     x: 0,
-                                     y: 0,
-                                     accessible_role: Atk.Role.DIALOG });
+        this._group = new St.Widget({ visible: false,
+                                      x: 0,
+                                      y: 0,
+                                      accessible_role: Atk.Role.DIALOG });
         Main.uiGroup.add_actor(this._group);
 
         let constraint = new Clutter.BindConstraint({ source: global.stage,
@@ -167,14 +167,17 @@ ModalDialog.prototype = {
      * ```
      */
     setButtons: function(buttons) {
-        let hadChildren = this._buttonLayout.get_children() > 0;
+        let hadChildren = this._buttonLayout.get_n_children() > 0;
 
-        this._buttonLayout.destroy_children();
+        this._buttonLayout.destroy_all_children();
         this._actionKeys = {};
         let focusSetExplicitly = false;
 
         for (let i = 0; i < buttons.length; i ++) {
             let buttonInfo = buttons[i];
+            if (!buttonInfo.focused) {
+                buttonInfo.focused = false;
+            }
             let label = buttonInfo['label'];
             let action = buttonInfo['action'];
             let key = buttonInfo['key'];
@@ -391,11 +394,11 @@ ModalDialog.prototype = {
      * can be dismissed by a close call.
      *
      * The main point of this method is to give some indication to the user
-     * that the dialog reponse has been acknowledged but will take a few
+     * that the dialog response has been acknowledged but will take a few
      * moments before being processed.
      *
      * e.g., if a user clicked "Log Out" then the dialog should go away
-     * imediately, but the lightbox should remain until the logout is
+     * immediately, but the lightbox should remain until the logout is
      * complete.
      */
     _fadeOutDialog: function(timestamp) {
@@ -446,14 +449,13 @@ SpicesAboutDialog.prototype = {
 
         //prepare translation
         this.uuid = metadata.uuid;
-        Gettext.bindtextdomain(metadata.uuid, GLib.get_home_dir() + "/.local/share/locale");
 
         let contentBox = new St.BoxLayout({vertical: true, style_class: "about-content" });
         this.contentLayout.add_actor(contentBox);
-        
+
         let topBox = new St.BoxLayout();
         contentBox.add_actor(topBox);
-        
+
         //icon
         let icon;
         if (metadata.icon) {
@@ -468,32 +470,42 @@ SpicesAboutDialog.prototype = {
             }
         }
         topBox.add_actor(icon);
-        
+
         let topTextBox = new St.BoxLayout({vertical: true});
         topBox.add_actor(topTextBox);
-        
-        /*title*/
-        let titleBox = new St.BoxLayout();
-        topTextBox.add_actor(titleBox);
 
+        /*title*/
         let title = new St.Label({text: this._(metadata.name), style_class: "about-title"});
-        titleBox.add_actor(title);
-        
-        if (metadata.version) {
-            let versionBin = new St.Bin({x_align: St.Align.START, y_align: St.Align.END});
-            titleBox.add_actor(versionBin);
-            let version = new St.Label({text: " v%s".format(metadata.version), style_class: "about-version"});
-            versionBin.add_actor(version);
-        }
-        
+        topTextBox.add_actor(title);
+
         //uuid
         let uuid = new St.Label({text: metadata.uuid, style_class: "about-uuid"});
         topTextBox.add_actor(uuid);
-        
+
+        //last-edited timestamp
+        if ('last-edited' in metadata) {
+            let lastEditedTimestamp = metadata['last-edited'];
+            let date = new Date(lastEditedTimestamp*1000);
+            let dateUTC = date.toISOString().replace(/T/, ' ');               // replace T with a space;
+            dateUTC = dateUTC.substring(0,dateUTC.lastIndexOf(':')) + ' UTC'; // remove seconds and append UTC label
+
+            let lastEdited = new St.Label({text: _("Last modified:") + " " + dateUTC, style_class: "about-uuid"});
+            topTextBox.add_actor(lastEdited);
+        }
+
+        //version
+        if (metadata.version) {
+            let version = new St.Label({text: _("Version:") + " " + "%s".format(metadata.version), style_class: "about-uuid"});
+            topTextBox.add_actor(version);
+        }
+
         //description
         let desc = new St.Label({text: this._(metadata.description), style_class: "about-description"});
         let dText = desc.clutter_text;
-        topTextBox.add_actor(desc);
+        dText.ellipsize = Pango.EllipsizeMode.NONE;
+        dText.line_wrap = true;
+        dText.set_line_wrap_mode(Pango.WrapMode.WORD_CHAR);
+        contentBox.add_actor(desc);
 
         // optional content
         if(metadata.comments || metadata.website || metadata.contributors){
@@ -504,7 +516,7 @@ SpicesAboutDialog.prototype = {
 
             // comments
             if (metadata.comments) {
-                let comments = new St.Label({text: _("Comments:") + "\n\t" + this._(metadata.comments)});
+                let comments = new St.Label({text: this._(metadata.comments)});
                 let cText = comments.clutter_text;
                 cText.ellipsize = Pango.EllipsizeMode.NONE;
                 cText.line_wrap = true;
@@ -522,7 +534,7 @@ SpicesAboutDialog.prototype = {
 
                 let wsButton = new St.Button({x_align: St.Align.START, style_class: "cinnamon-link", name: "about-website"});
                 wsBox.add_actor(wsButton);
-                let website = new St.Label({text: metadata.website});
+                let website = new St.Label({text: "\t" + metadata.website});
                 let wtext = website.clutter_text;
                 wtext.ellipsize = Pango.EllipsizeMode.NONE;
                 wtext.line_wrap = true;
@@ -546,12 +558,21 @@ SpicesAboutDialog.prototype = {
                 infoBox.add_actor(contributors);
             }
         }
-        
-        //dialog close button
-        this.setButtons([
-            {label: _("Close"), key: "", focus: true, action: Lang.bind(this, this._onOk)}
-        ]);
-        
+
+        //dialog buttons, if it's a spice, add a "More info" button
+        let spicesID = this._getSpicesID(metadata.uuid, type);
+        if (spicesID) {
+            let spicesWebsite = "http://cinnamon-spices.linuxmint.com/" + type + "/view/" + spicesID;
+            this.setButtons([
+                {label: _("More info"), key: "", focus: true, action: Lang.bind(this, this._launchSite, spicesWebsite)},
+                {label: _("Close"), key: "", focus: true, action: Lang.bind(this, this._onOk)}
+            ]);
+        } else {
+            this.setButtons([
+                {label: _("Close"), key: "", focus: true, action: Lang.bind(this, this._onOk)}
+            ]);
+        }
+
         this.open(global.get_current_time());
     },
 
@@ -567,10 +588,34 @@ SpicesAboutDialog.prototype = {
         return _(str);
     },
 
+    _getSpicesID: function(uuid, type) {
+        try {
+            let indexCacheFile = Gio.file_new_for_path(GLib.get_home_dir() + "/.cinnamon/spices.cache/" + type.slice(0, -1) + "/index.json");
+            if(this._ensureFileExists(indexCacheFile)) {
+                let indexCacheContents = Cinnamon.get_file_contents_utf8_sync(indexCacheFile.get_path());
+                let index_cache = JSON.parse(indexCacheContents);
+                if (uuid in index_cache)
+                    return index_cache[uuid]['spices-id'];
+            }
+            return null;
+        } catch (e) {
+            global.log('Failed to load/parse index.json', e);
+            return null;
+        }
+    },
+
+    _ensureFileExists: function(file) {
+        if (!file.query_exists(null)) {
+            global.log('File not found: ' + file.get_path());
+            return false;
+        }
+        return true;
+    },
+
     _onOk: function() {
         this.close(global.get_current_time());
     },
-    
+
     _launchSite: function(a, b, site) {
         Util.spawnCommandLine("xdg-open " + site);
         this.close(global.get_current_time());
@@ -698,7 +743,7 @@ InfoOSD.prototype = {
      * show:
      * @monitorIndex (int): (optional) Monitor to display OSD on. Default is
      * primary monitor
-     * 
+     *
      * Shows the OSD at the center of monitor @monitorIndex. Shows at the
      * primary monitor if not specified.
      */
@@ -728,7 +773,7 @@ InfoOSD.prototype = {
 
     /**
      * destroy:
-     * 
+     *
      * Destroys the OSD
      */
     destroy: function() {
