@@ -207,7 +207,7 @@ NotificationDaemon.prototype = {
         if (ndata && ndata.notification)
             return ndata.notification.source;
 
-        let isForTransientNotification = (ndata && ndata.hints['transient'] == true);
+        let isForTransientNotification = (ndata && ndata.hints.maybeGet('transient') == true);
 
         // We don't want to override a persistent notification
         // with a transient one from the same sender, so we
@@ -255,10 +255,14 @@ NotificationDaemon.prototype = {
          this._startExpire();
     },
     _expireNotification: function() {
-         let ndata = this._expireNotifications[0];
-         ndata.notification.destroy(MessageTray.NotificationDestroyedReason.EXPIRED);
-         this._expireTimer = 0;
-         return false;
+        let ndata = this._expireNotifications[0];
+
+        if (ndata) {
+            ndata.notification.destroy(MessageTray.NotificationDestroyedReason.EXPIRED);
+        }
+
+        this._expireTimer = 0;
+        return false;
     },
  
     // Sends a notification to the notification daemon. Returns the id allocated to the notification.
@@ -457,7 +461,7 @@ NotificationDaemon.prototype = {
         }
 
         if (actions.length) {
-            notification.setUseActionIcons(hints['action-icons'] == true);
+            notification.setUseActionIcons(hints.maybeGet('action-icons') == true);
             for (let i = 0; i < actions.length - 1; i += 2) {
                 if (actions[i] == 'default')
                     notification.connect('clicked', Lang.bind(this,
@@ -479,10 +483,10 @@ NotificationDaemon.prototype = {
                 notification.setUrgency(MessageTray.Urgency.CRITICAL);
                 break;
         }
-        notification.setResident(hints.resident == true);
+        notification.setResident(hints.maybeGet('resident') == true);
         // 'transient' is a reserved keyword in JS, so we have to retrieve the value
         // of the 'transient' hint with hints['transient'] rather than hints.transient
-        notification.setTransient(hints['transient'] == true);
+        notification.setTransient(hints.maybeGet('transient') == true);
 
         let sourceIconActor = source.useNotificationIcon ? this._iconForNotificationData(icon, hints, source.ICON_SIZE) : null;
         source.processNotification(notification, sourceIconActor);
