@@ -29,8 +29,8 @@ Overview.prototype = {
     _init : function() {
         this._spacing = 0;
 
-        this._group = new St.Group({ name: 'overview',
-                                     reactive: true });
+        this._group = new St.Widget({ name: 'overview',
+                                      reactive: true });
         this._group._delegate = this;
         this._group.connect('style-changed',
             Lang.bind(this, function() {
@@ -433,8 +433,11 @@ Overview.prototype = {
     _hideDone: function() {
         this._group.remove_actor(this._coverPane);
         this._coverPane.destroy();
+        this._coverPane = null;
+
         global.overlay_group.remove_actor(this._background);
         this._background.destroy();
+        this._background = null;
 
         // Re-enable unredirection
         Meta.enable_unredirect_for_screen(global.screen);
