@@ -509,7 +509,12 @@ WindowManager.prototype = {
             return global.settings.get_boolean("desktop-effects");
         }
         if (type == Meta.WindowType.DIALOG || type == Meta.WindowType.MODAL_DIALOG) {
-            return global.settings.get_boolean("desktop-effects-on-dialogs");
+            return global.settings.get_boolean("desktop-effects") && global.settings.get_boolean("desktop-effects-on-dialogs");
+        }
+        if (type == Meta.WindowType.MENU ||
+            type == Meta.WindowType.DROPDOWN_MENU ||
+            type == Meta.WindowType.POPUP_MENU) {
+            return global.settings.get_boolean("desktop-effects") && global.settings.get_boolean("desktop-effects-on-menus");
         }
         return false;
     },
@@ -557,7 +562,7 @@ WindowManager.prototype = {
     },
 
     _killWindowEffects: function (cinnamonwm, actor) {
-        for(let i in this._effects){
+        for(let i in this.effects){
             this._endWindowEffect(cinnamonwm, i, actor);
         }
     },
@@ -671,7 +676,7 @@ WindowManager.prototype = {
             this._checkDimming(actor.get_meta_window().get_transient_for());
         }
 
-        if (actor.get_meta_window()._cinnamonwm_has_origin === true) {
+        if (actor.get_meta_window()._cinnamonwm_has_origin && actor.get_meta_window()._cinnamonwm_has_origin === true) {
             Main.soundManager.play('minimize');
             try {
                 this._startWindowEffect(cinnamonwm, "unminimize", actor, null, "minimize")
@@ -720,10 +725,14 @@ WindowManager.prototype = {
 
     _switchWorkspace : function(cinnamonwm, from, to, direction) {
         if (!this._shouldAnimate()) {
+            this.showWorkspaceOSD();
             cinnamonwm.completed_switch_workspace();
             return;
         }
 
+        Main.soundManager.play('switch');
+        this.showWorkspaceOSD();
+
         let windows = global.get_window_actors();
 
         /* @direction is the direction that the "camera" moves, so the
@@ -948,7 +957,6 @@ WindowManager.prototype = {
         if (workspace != global.screen.get_active_workspace()) {
             window.change_workspace(workspace);
             workspace.activate_with_focus(window, global.get_current_time());
-            this.showWorkspaceOSD();
         }
     },
 
@@ -960,6 +968,16 @@ WindowManager.prototype = {
         this._shiftWindowToWorkspace(window, Meta.MotionDirection.RIGHT);
     },
 
+    moveToWorkspace: function(workspace, direction_hint) {
+        let active = global.screen.get_active_workspace();
+        if (workspace != active) {
+            if (direction_hint)
+                workspace.activate_with_direction_hint(direction_hint, global.get_current_time());
+            else
+                workspace.activate(global.get_current_time());
+        }
+    },
+
     _showWorkspaceSwitcher : function(display, screen, window, binding) {
         if (binding.get_name() == 'switch-to-workspace-up') {
             Main.expo.toggle();
@@ -973,36 +991,26 @@ WindowManager.prototype = {
         if (screen.n_workspaces == 1)
             return;
 
-        let current_workspace_index = global.screen.get_active_workspace_index();
         if (binding.get_name() == 'switch-to-workspace-left') {
            this.actionMoveWorkspaceLeft();
-           if (current_workspace_index !== global.screen.get_active_workspace_index()) {
-                this.showWorkspaceOSD();
-           }
-        }
-        else if (binding.get_name() == 'switch-to-workspace-right') {
+        } else if (binding.get_name() == 'switch-to-workspace-right') {
            this.actionMoveWorkspaceRight();
-           if (current_workspace_index !== global.screen.get_active_workspace_index()) {
-                this.showWorkspaceOSD();
-           }
         }
     },
 
     actionMoveWorkspaceLeft: function() {
-        var active = global.screen.get_active_workspace();
-        var neighbour = active.get_neighbor(Meta.MotionDirection.LEFT)
-        if (active != neighbour) {
-            Main.soundManager.play('switch');
-            neighbour.activate(global.get_current_time());
+        let active = global.screen.get_active_workspace();
+        let neighbor = active.get_neighbor(Meta.MotionDirection.LEFT)
+        if (active != neighbor) {
+            this.moveToWorkspace(neighbor, Meta.MotionDirection.LEFT);
         }
     },
 
     actionMoveWorkspaceRight: function() {
-        var active = global.screen.get_active_workspace();
-        var neighbour = active.get_neighbor(Meta.MotionDirection.RIGHT)
-        if (active != neighbour) {
-            Main.soundManager.play('switch');
-            neighbour.activate(global.get_current_time());
+        let active = global.screen.get_active_workspace();
+        let neighbor = active.get_neighbor(Meta.MotionDirection.RIGHT)
+        if (active != neighbor) {
+            this.moveToWorkspace(neighbor, Meta.MotionDirection.RIGHT);
         }
     },
 
