@@ -39,7 +39,7 @@ typedef struct {
   GSList *windows;
 
   /* Whether or not we need to resort the windows; this is done on demand */
-  gboolean window_sort_stale : 1;
+  guint window_sort_stale : 1;
 } CinnamonAppRunningState;
 
 /**
@@ -123,12 +123,45 @@ cinnamon_app_get_id (CinnamonApp *app)
 static MetaWindow *
 window_backed_app_get_window (CinnamonApp     *app)
 {
-  g_assert (app->entry == NULL);
   g_assert (app->running_state);
   g_assert (app->running_state->windows);
   return app->running_state->windows->data;
 }
 
+static ClutterActor *
+get_actor_for_icon_name (CinnamonApp *app,
+                         const gchar *icon_name,
+                         gint         size)
+{
+  ClutterActor *actor;
+  GIcon *icon;
+
+  icon = NULL;
+  actor = NULL;
+
+  if (g_path_is_absolute (icon_name))
+    {
+      GFile *icon_file;
+
+      icon_file = g_file_new_for_path (icon_name);
+      icon = g_file_icon_new (icon_file);
+
+      g_object_unref (icon_file);
+    }
+  else
+    {
+      icon = g_themed_icon_new (icon_name);
+    }
+
+  if (icon != NULL)
+  {
+    actor = g_object_new (ST_TYPE_ICON, "gicon", icon, "icon-size", size, NULL);
+    g_object_unref (icon);
+  }
+
+  return actor;
+}
+
 static ClutterActor *
 window_backed_app_get_icon (CinnamonApp *app,
                             int       size)
@@ -138,13 +171,14 @@ window_backed_app_get_icon (CinnamonApp *app,
   gint scale;
   CinnamonGlobal *global;
   StThemeContext *context;
+  const gchar *icon_name;
+
+  actor = NULL;
 
   global = cinnamon_global_get ();
   context = st_theme_context_get_for_stage (cinnamon_global_get_stage (global));
   g_object_get (context, "scale-factor", &scale, NULL);
 
-  size *= scale;
-
   /* During a state transition from running to not-running for
    * window-backend apps, it's possible we get a request for the icon.
    * Avoid asserting here and just return an empty image.
@@ -157,10 +191,23 @@ window_backed_app_get_icon (CinnamonApp *app,
     }
 
   window = window_backed_app_get_window (app);
-  actor = st_texture_cache_bind_pixbuf_property (st_texture_cache_get_default (),
-                                                               G_OBJECT (window),
-                                                               "icon");
-  g_object_set (actor, "width", (float) size, "height", (float) size, NULL);
+
+  icon_name = meta_window_get_icon_name (window);
+
+  if (icon_name != NULL)
+    {
+      actor = get_actor_for_icon_name (app, icon_name, size);
+    }
+
+  if (actor == NULL)
+    {
+      size *= scale;
+
+      actor = st_texture_cache_bind_pixbuf_property (st_texture_cache_get_default (),
+                                                     G_OBJECT (window), "icon");
+      g_object_set (actor, "width", (float) size, "height", (float) size, NULL);
+    }
+
   return actor;
 }
 
@@ -178,20 +225,39 @@ cinnamon_app_create_icon_texture (CinnamonApp   *app,
 {
   GIcon *icon;
   ClutterActor *ret;
+  gboolean has_custom_icon;
 
+  has_custom_icon = FALSE;
   ret = NULL;
 
-  if (app->entry == NULL)
-    return window_backed_app_get_icon (app, size);
+  if (app->running_state != NULL)
+  {
+    MetaWindow *window;
+    const gchar *icon_name;
+
+    window = window_backed_app_get_window (app);
+
+    icon_name = meta_window_get_icon_name (window);
+
+    has_custom_icon = icon_name != NULL;
+  }
+
+  if (app->entry == NULL || has_custom_icon)
+    {
+      return window_backed_app_get_icon (app, size);
+    }
 
   icon = g_app_info_get_icon (G_APP_INFO (gmenu_tree_entry_get_app_info (app->entry)));
+
   if (icon != NULL)
-    ret = st_texture_cache_load_gicon (st_texture_cache_get_default (), NULL, icon, size);
+    {
+      ret = g_object_new (ST_TYPE_ICON, "gicon", icon, "icon-size", size, NULL);
+    }
 
   if (ret == NULL)
     {
       icon = g_themed_icon_new ("application-x-executable");
-      ret = st_texture_cache_load_gicon (st_texture_cache_get_default (), NULL, icon, size);
+      ret = g_object_new (ST_TYPE_ICON, "gicon", icon, "icon-size", size, NULL);
       g_object_unref (icon);
     }
 
@@ -221,7 +287,7 @@ cinnamon_app_create_faded_icon_cpu (StTextureCache *cache,
   gboolean have_alpha;
   gint fade_start;
   gint fade_range;
-  guint i, j;
+  int i, j;
   guint pixbuf_byte_size;
   guint8 *orig_pixels;
   guint8 *pixels;
@@ -424,7 +490,7 @@ cinnamon_app_get_keywords (CinnamonApp *app)
  * cinnamon_app_is_window_backed:
  *
  * A window backed application is one which represents just an open
- * window, i.e. there's no .desktop file assocation, so we don't know
+ * window, i.e. there's no .desktop file association, so we don't know
  * how to launch it again.
  */
 gboolean
@@ -565,7 +631,7 @@ cinnamon_app_activate_window (CinnamonApp     *app,
         window = most_recent_transient;
 
 
-      if (!cinnamon_window_tracker_is_window_interesting (window))
+      if (!cinnamon_window_tracker_is_window_interesting (cinnamon_window_tracker_get_default (), window))
         {
           /* We won't get notify::user-time signals for uninteresting windows,
            * which means that an app's last_user_time won't get updated.
@@ -646,6 +712,9 @@ cinnamon_app_activate_full (CinnamonApp      *app,
       case CINNAMON_APP_STATE_RUNNING:
         cinnamon_app_activate_window (app, NULL, timestamp);
         break;
+      default:
+        g_warning("cinnamon_app_activate_full: default case");
+        break;
     }
 }
 
@@ -1010,7 +1079,7 @@ _cinnamon_app_remove_window (CinnamonApp   *app,
  * cinnamon_app_get_pids:
  * @app: a #CinnamonApp
  *
- * Returns: (transfer container) (element-type int): An unordered list of process identifers associated with this application.
+ * Returns: (transfer container) (element-type int): An unordered list of process identifiers associated with this application.
  */
 GSList *
 cinnamon_app_get_pids (CinnamonApp *app)
@@ -1079,6 +1148,7 @@ _cinnamon_app_handle_startup_sequence (CinnamonApp          *app,
 gboolean
 cinnamon_app_request_quit (CinnamonApp   *app)
 {
+  CinnamonGlobal *global;
   GSList *iter;
 
   if (cinnamon_app_get_state (app) != CINNAMON_APP_STATE_RUNNING)
@@ -1086,14 +1156,16 @@ cinnamon_app_request_quit (CinnamonApp   *app)
 
   /* TODO - check for an XSMP connection; we could probably use that */
 
+  global = cinnamon_global_get ();
+
   for (iter = app->running_state->windows; iter; iter = iter->next)
     {
       MetaWindow *win = iter->data;
 
-      if (!cinnamon_window_tracker_is_window_interesting (win))
+      if (!cinnamon_window_tracker_is_window_interesting (cinnamon_window_tracker_get_default (), win))
         continue;
 
-      meta_window_delete (win, cinnamon_global_get_current_time (cinnamon_global_get ()));
+      meta_window_delete (win, cinnamon_global_get_current_time (global));
     }
   return TRUE;
 }
@@ -1380,6 +1452,9 @@ _cinnamon_app_do_match (CinnamonApp         *app,
       case MATCH_SUBSTRING:
         *substring_results = g_slist_prepend (*substring_results, app);
         break;
+      default:
+        g_warning("cinnamon_app_do_match: default case");
+        break;
     }
 }
 
@@ -1402,11 +1477,8 @@ cinnamon_app_dispose (GObject *object)
       app->entry = NULL;
     }
 
-  if (app->running_state)
-    {
-      while (app->running_state->windows)
-        _cinnamon_app_remove_window (app, app->running_state->windows->data);
-    }
+  while (app->running_state)
+    _cinnamon_app_remove_window (app, app->running_state->windows->data);
 
   g_clear_pointer (&app->keywords, g_free);
 
@@ -1441,8 +1513,7 @@ cinnamon_app_class_init(CinnamonAppClass *klass)
                                      CINNAMON_TYPE_APP,
                                      G_SIGNAL_RUN_LAST,
                                      0,
-                                     NULL, NULL,
-                                     g_cclosure_marshal_VOID__VOID,
+                                     NULL, NULL, NULL,
                                      G_TYPE_NONE, 0);
 
   /**
