@@ -443,7 +443,7 @@ st_entry_allocate (ClutterActor          *actor,
                               flags);
 
       /* reduce the size for the entry */
-      child_box.x2 -= icon_w - priv->spacing;
+      child_box.x2 -= icon_w + priv->spacing;
     }
 
   clutter_actor_get_preferred_height (priv->entry, child_box.x2 - child_box.x1,
@@ -530,7 +530,7 @@ blink_cb (gpointer data)
 {
   StEntry *entry;
   StEntryPrivate *priv;
-  gint blink_timeout;
+  guint blink_timeout;
 
   entry = ST_ENTRY (data);
   priv = entry->priv;
@@ -733,6 +733,41 @@ st_entry_clipboard_callback (StClipboard *clipboard,
   clutter_text_insert_text (ctext, text, cursor_pos);
 }
 
+static gboolean
+clutter_text_button_press_event (ClutterActor       *actor,
+                                 ClutterButtonEvent *event,
+                                 gpointer            user_data)
+{
+  StEntryPrivate *priv = ST_ENTRY_PRIV (user_data);
+  GtkSettings *settings = gtk_settings_get_default ();
+  gboolean primary_paste_enabled;
+
+  g_object_get (settings,
+                "gtk-enable-primary-paste", &primary_paste_enabled,
+                NULL);
+
+  if (primary_paste_enabled && event->button == 2
+      && clutter_text_get_editable (CLUTTER_TEXT (priv->entry)))
+    {
+      StClipboard *clipboard;
+
+      clipboard = st_clipboard_get_default ();
+
+      /* By the time the clipboard callback is called,
+       * the rest of the signal handlers will have
+       * run, making the text cursor to be in the correct
+       * place.
+       */
+      st_clipboard_get_text (clipboard,
+                             ST_CLIPBOARD_TYPE_PRIMARY,
+                             st_entry_clipboard_callback,
+                             user_data);
+    }
+
+  return FALSE;
+}
+
+
 static gboolean
 st_entry_key_press_event (ClutterActor    *actor,
                           ClutterKeyEvent *event)
@@ -748,20 +783,23 @@ st_entry_key_press_event (ClutterActor    *actor,
 
   /* paste */
   if ((event->modifier_state & CLUTTER_CONTROL_MASK)
-      && event->keyval == CLUTTER_v)
+      && (event->keyval == CLUTTER_KEY_v || event->keyval == CLUTTER_KEY_V))
     {
       StClipboard *clipboard;
 
       clipboard = st_clipboard_get_default ();
 
-      st_clipboard_get_text (clipboard, st_entry_clipboard_callback, actor);
+      st_clipboard_get_text (clipboard,
+                             ST_CLIPBOARD_TYPE_CLIPBOARD,
+                             st_entry_clipboard_callback,
+                             actor);
 
       return TRUE;
     }
 
   /* copy */
   if ((event->modifier_state & CLUTTER_CONTROL_MASK)
-      && event->keyval == CLUTTER_c)
+      && (event->keyval == CLUTTER_KEY_c || event->keyval == CLUTTER_KEY_C))
     {
       StClipboard *clipboard;
       gchar *text;
@@ -771,7 +809,9 @@ st_entry_key_press_event (ClutterActor    *actor,
       text = clutter_text_get_selection ((ClutterText*) priv->entry);
 
       if (text && strlen (text))
-        st_clipboard_set_text (clipboard, text);
+        st_clipboard_set_text (clipboard,
+                               ST_CLIPBOARD_TYPE_CLIPBOARD,
+                               text);
 
       return TRUE;
     }
@@ -779,7 +819,7 @@ st_entry_key_press_event (ClutterActor    *actor,
 
   /* cut */
   if ((event->modifier_state & CLUTTER_CONTROL_MASK)
-      && event->keyval == CLUTTER_x)
+      && (event->keyval == CLUTTER_KEY_x || event->keyval == CLUTTER_KEY_X))
     {
       StClipboard *clipboard;
       gchar *text;
@@ -790,7 +830,7 @@ st_entry_key_press_event (ClutterActor    *actor,
 
       if (text && strlen (text))
         {
-          st_clipboard_set_text (clipboard, text);
+          st_clipboard_set_text (clipboard,ST_CLIPBOARD_TYPE_CLIPBOARD, text);
 
           /* now delete the text */
           clutter_text_delete_selection ((ClutterText *) priv->entry);
@@ -908,6 +948,9 @@ st_entry_init (StEntry *entry)
   g_signal_connect (priv->entry, "notify::password-char",
                     G_CALLBACK (clutter_text_password_char_cb), entry);
 
+  g_signal_connect (priv->entry, "button-press-event",
+                    G_CALLBACK (clutter_text_button_press_event), entry);
+
   g_signal_connect (priv->entry, "notify::selection-bound",
                     G_CALLBACK (clutter_text_selection_bound_cb), entry);
 
