@@ -188,12 +188,6 @@ st_label_dispose (GObject   *object)
 {
   StLabelPrivate *priv = ST_LABEL (object)->priv;
 
-  if (priv->label)
-    {
-      clutter_actor_destroy (priv->label);
-      priv->label = NULL;
-    }
-
   if (priv->text_shadow_material != COGL_INVALID_HANDLE)
     {
       cogl_handle_unref (priv->text_shadow_material);
@@ -220,10 +214,6 @@ st_label_paint (ClutterActor *actor)
       clutter_actor_get_allocation_box (priv->label, &allocation);
       clutter_actor_box_get_size (&allocation, &width, &height);
 
-      allocation.x1 = allocation.y1 = 0;
-      allocation.x2 = width;
-      allocation.y2 = height;
-
       if (priv->text_shadow_material == COGL_INVALID_HANDLE ||
           width != priv->shadow_width ||
           height != priv->shadow_height)
@@ -414,7 +404,7 @@ st_label_set_text (StLabel     *label,
 ClutterActor*
 st_label_get_clutter_text (StLabel *label)
 {
-  g_return_val_if_fail (ST_LABEL (label), NULL);
+  g_return_val_if_fail (ST_IS_LABEL (label), NULL);
 
   return label->priv->label;
 }
@@ -459,9 +449,6 @@ struct _StLabelAccessibleClass
   StWidgetAccessibleClass parent_class;
 };
 
-static void st_label_accessible_class_init (StLabelAccessibleClass *klass);
-static void st_label_accessible_init       (StLabelAccessible *label);
-
 /* AtkObject */
 static void          st_label_accessible_initialize (AtkObject *obj,
                                                      gpointer   data);
