@@ -59,7 +59,6 @@
  */
 
 #include "st-scroll-view.h"
-#include "st-marshal.h"
 #include "st-scroll-bar.h"
 #include "st-scrollable.h"
 #include "st-scroll-view-fade.h"
@@ -82,9 +81,10 @@ G_DEFINE_TYPE_WITH_CODE (StScrollView, st_scroll_view, ST_TYPE_BIN,
 
 #define AUTO_SCROLL_TOTAL_REGION 100
 
-#define AUTO_SCROLL_OVERLAP 10 /* autoscroll region extends this far into the scroll view
-                                * remaining portion of region is above or below the view
-                                */
+/* autoscroll region extends this far into the scroll view
+ * remaining portion of region is above or below the view
+ */
+#define AUTO_SCROLL_OVERLAP 10
 
 /* gdouble - from 0 to TOTAL_REGION / this number to get scroll delta */
 #define AUTO_SCROLL_SPEED_DIVISOR 4.0
@@ -114,12 +114,12 @@ struct _StScrollViewPrivate
 
   StScrollViewFade *vfade_effect;
 
-  gboolean      row_size_set : 1;
-  gboolean      column_size_set : 1;
+  guint         row_size_set : 1;
+  guint         column_size_set : 1;
   guint         mouse_scroll : 1;
   guint         hscrollbar_visible : 1;
   guint         vscrollbar_visible : 1;
-  gboolean      auto_scroll : 1;
+  guint         auto_scroll : 1;
   guint         auto_scroll_timeout_id;
 };
 
@@ -257,6 +257,8 @@ calculate_and_scroll (ClutterActor  *self,
                     "value", &vvalue,
                     NULL);
     st_adjustment_set_value (priv->vadjustment, vvalue + delta);
+
+    clutter_actor_queue_redraw (self);
 }
 
 static void
@@ -421,13 +423,13 @@ st_scroll_view_paint (ClutterActor *actor)
 {
   StScrollViewPrivate *priv = ST_SCROLL_VIEW (actor)->priv;
 
-  /* StBin will paint the child */
-  CLUTTER_ACTOR_CLASS (st_scroll_view_parent_class)->paint (actor);
+  st_widget_paint_background (ST_WIDGET (actor));
 
-  /* paint our custom children */
-  if (priv->hscrollbar_visible && CLUTTER_ACTOR_IS_VISIBLE (priv->hscroll))
+  if (priv->child)
+    clutter_actor_paint (priv->child);
+  if (priv->hscrollbar_visible)
     clutter_actor_paint (priv->hscroll);
-  if (priv->vscrollbar_visible && CLUTTER_ACTOR_IS_VISIBLE (priv->vscroll))
+  if (priv->vscrollbar_visible)
     clutter_actor_paint (priv->vscroll);
 }
 
@@ -440,10 +442,11 @@ st_scroll_view_pick (ClutterActor       *actor,
   /* Chain up so we get a bounding box pained (if we are reactive) */
   CLUTTER_ACTOR_CLASS (st_scroll_view_parent_class)->pick (actor, color);
 
-  /* paint our custom children */
-  if (priv->hscrollbar_visible && CLUTTER_ACTOR_IS_VISIBLE (priv->hscroll))
+  if (priv->child)
+    clutter_actor_paint (priv->child);
+  if (priv->hscrollbar_visible)
     clutter_actor_paint (priv->hscroll);
-  if (priv->vscrollbar_visible && CLUTTER_ACTOR_IS_VISIBLE (priv->vscroll))
+  if (priv->vscrollbar_visible)
     clutter_actor_paint (priv->vscroll);
 }
 
@@ -453,7 +456,7 @@ get_scrollbar_width (StScrollView *scroll,
 {
   StScrollViewPrivate *priv = scroll->priv;
 
-  if (CLUTTER_ACTOR_IS_VISIBLE (priv->vscroll))
+  if (clutter_actor_is_visible (priv->vscroll))
     {
       gfloat min_size;
 
@@ -471,7 +474,7 @@ get_scrollbar_height (StScrollView *scroll,
 {
   StScrollViewPrivate *priv = scroll->priv;
 
-  if (CLUTTER_ACTOR_IS_VISIBLE (priv->hscroll))
+  if (clutter_actor_is_visible (priv->hscroll))
     {
       gfloat min_size;
 
@@ -516,6 +519,9 @@ st_scroll_view_get_preferred_width (ClutterActor *actor,
        * but that's not cleanly defined at the moment */
       min_width = 0;
       break;
+    default:
+      g_warn_if_reached();
+      break;
     }
 
   if (priv->vscrollbar_policy != GTK_POLICY_NEVER)
@@ -571,6 +577,9 @@ st_scroll_view_get_preferred_height (ClutterActor *actor,
       /* We've requested space for the scrollbar, subtract it back out */
       for_width -= sb_width;
       break;
+    default:
+      g_warn_if_reached();
+      break;
     }
 
   switch (priv->hscrollbar_policy)
@@ -584,6 +593,9 @@ st_scroll_view_get_preferred_height (ClutterActor *actor,
     case GTK_POLICY_AUTOMATIC:
       account_for_hscrollbar = for_width < child_min_width;
       break;
+    default:
+      g_warn_if_reached();
+      break;
     }
 
   clutter_actor_get_preferred_height (priv->child, for_width,
@@ -602,6 +614,9 @@ st_scroll_view_get_preferred_height (ClutterActor *actor,
        * but that's not cleanly defined at the moment */
       min_height = 0;
       break;
+    default:
+      g_warn_if_reached();
+      break;
     }
 
   if (account_for_hscrollbar)
@@ -627,24 +642,13 @@ st_scroll_view_allocate (ClutterActor          *actor,
                          ClutterAllocationFlags flags)
 {
   ClutterActorBox content_box, child_box;
-  ClutterActorClass *parent_parent_class;
   gfloat avail_width, avail_height, sb_width, sb_height;
   gboolean hscrollbar_visible, vscrollbar_visible;
 
   StScrollViewPrivate *priv = ST_SCROLL_VIEW (actor)->priv;
   StThemeNode *theme_node = st_widget_get_theme_node (ST_WIDGET (actor));
 
-  /* Chain up to the parent's parent class
-   *
-   * We do this because we do not want StBin to allocate the child, as we
-   * give it a different allocation later, depending on whether the scrollbars
-   * are visible
-   */
-  parent_parent_class
-    = g_type_class_peek_parent (st_scroll_view_parent_class);
-
-  CLUTTER_ACTOR_CLASS (parent_parent_class)->
-  allocate (actor, box, flags);
+  clutter_actor_set_allocation (actor, box, flags);
 
   st_theme_node_get_content_box (theme_node, box, &content_box);
 
@@ -727,7 +731,7 @@ st_scroll_view_allocate (ClutterActor          *actor,
    */
 
   /* Vertical scrollbar */
-  if (CLUTTER_ACTOR_IS_VISIBLE (priv->vscroll))
+  if (clutter_actor_is_visible (priv->vscroll))
     {
       if (st_widget_get_direction (ST_WIDGET (actor)) == ST_TEXT_DIRECTION_RTL)
         {
@@ -746,7 +750,7 @@ st_scroll_view_allocate (ClutterActor          *actor,
     }
 
   /* Horizontal scrollbar */
-  if (CLUTTER_ACTOR_IS_VISIBLE (priv->hscroll))
+  if (clutter_actor_is_visible (priv->hscroll))
     {
       if (st_widget_get_direction (ST_WIDGET (actor)) == ST_TEXT_DIRECTION_RTL)
         {
@@ -875,6 +879,9 @@ st_scroll_view_scroll_event (ClutterActor       *self,
                     "value", &value,
                     NULL);
       break;
+    default:
+      g_warn_if_reached();
+      break;
     }
 
   switch (event->direction)
@@ -895,8 +902,13 @@ st_scroll_view_scroll_event (ClutterActor       *self,
     case CLUTTER_SCROLL_RIGHT:
       st_adjustment_set_value (priv->hadjustment, value + step);
       break;
+    default:
+      g_warn_if_reached();
+      break;
     }
 
+  clutter_actor_queue_redraw (self);
+
   return TRUE;
 }
 
@@ -1080,7 +1092,7 @@ st_scroll_view_remove (ClutterContainer *container,
       else
         g_assert ("Unknown child removed from StScrollView");
 
-      clutter_actor_unparent (actor);
+      clutter_actor_remove_child (CLUTTER_ACTOR (container), actor);
     }
 }
 
