@@ -40,6 +40,7 @@
 #include "st-texture-cache.h"
 #include "st-theme-context.h"
 #include "st-theme-node-transition.h"
+#include "st-theme-node-private.h"
 
 #include "st-widget-accessible.h"
 
@@ -59,14 +60,13 @@ struct _StWidgetPrivate
 
   StThemeNodeTransition *transition_animation;
 
-  gboolean      is_stylable : 1;
-  gboolean      is_style_dirty : 1;
-  gboolean      draw_bg_color : 1;
-  gboolean      draw_border_internal : 1;
-  gboolean      track_hover : 1;
-  gboolean      hover : 1;
-  gboolean      can_focus : 1;
-  gboolean      important : 1;
+  guint      is_style_dirty : 1;
+  guint      draw_bg_color : 1;
+  guint      draw_border_internal : 1;
+  guint      track_hover : 1;
+  guint      hover : 1;
+  guint      can_focus : 1;
+  guint      important : 1;
 
   StTextDirection   direction;
 
@@ -103,7 +103,6 @@ enum
   PROP_PSEUDO_CLASS,
   PROP_STYLE_CLASS,
   PROP_STYLE,
-  PROP_STYLABLE,
   PROP_TRACK_HOVER,
   PROP_HOVER,
   PROP_CAN_FOCUS,
@@ -163,14 +162,6 @@ st_widget_set_property (GObject      *gobject,
       st_widget_set_style (actor, g_value_get_string (value));
       break;
 
-    case PROP_STYLABLE:
-      if (actor->priv->is_stylable != g_value_get_boolean (value))
-        {
-          actor->priv->is_stylable = g_value_get_boolean (value);
-          clutter_actor_queue_relayout ((ClutterActor *) gobject);
-        }
-      break;
-
     case PROP_TRACK_HOVER:
       st_widget_set_track_hover (actor, g_value_get_boolean (value));
       break;
@@ -232,10 +223,6 @@ st_widget_get_property (GObject    *gobject,
       g_value_set_string (value, priv->inline_style);
       break;
 
-    case PROP_STYLABLE:
-      g_value_set_boolean (value, priv->is_stylable);
-      break;
-
     case PROP_TRACK_HOVER:
       g_value_set_boolean (value, priv->track_hover);
       break;
@@ -287,25 +274,12 @@ st_widget_dispose (GObject *gobject)
   StWidget *actor = ST_WIDGET (gobject);
   StWidgetPrivate *priv = ST_WIDGET (actor)->priv;
 
-  if (priv->theme)
-    {
-      g_object_unref (priv->theme);
-      priv->theme = NULL;
-    }
-
-  if (priv->theme_node)
-    {
-      g_object_unref (priv->theme_node);
-      priv->theme_node = NULL;
-    }
+  g_clear_pointer (&priv->theme, g_object_unref);
+  g_clear_pointer (&priv->theme_node, g_object_unref);
 
   st_widget_remove_transition (actor);
 
-  if (priv->label_actor)
-    {
-      g_object_unref (priv->label_actor);
-      priv->label_actor = NULL;
-    }
+  g_clear_pointer (&priv->label_actor, g_object_unref);
 
   g_clear_object (&priv->prev_first_child);
   g_clear_object (&priv->prev_last_child);
@@ -496,12 +470,6 @@ notify_children_of_style_change (ClutterActor *self)
 static void
 st_widget_real_style_changed (StWidget *self)
 {
-  StWidgetPrivate *priv = ST_WIDGET (self)->priv;
-
-  /* application has request this widget is not stylable */
-  if (!priv->is_stylable)
-    return;
-
   clutter_actor_queue_redraw ((ClutterActor *) self);
 
   notify_children_of_style_change ((ClutterActor *) self);
@@ -520,7 +488,7 @@ st_widget_style_changed (StWidget *widget)
     }
 
   /* update the style only if we are mapped */
-  if (CLUTTER_ACTOR_IS_MAPPED (CLUTTER_ACTOR (widget)))
+  if (clutter_actor_is_mapped (CLUTTER_ACTOR (widget)))
     st_widget_recompute_style (widget, old_theme_node);
 
   if (old_theme_node)
@@ -606,9 +574,9 @@ st_widget_get_theme_node (StWidget *widget)
        * requiring separate style sheets.
        */
       if (st_widget_get_direction (widget) == ST_TEXT_DIRECTION_RTL)
-        direction_pseudo_class = "rtl";
+        direction_pseudo_class = (char *)"rtl";
       else
-        direction_pseudo_class = "ltr";
+        direction_pseudo_class = (char *)"ltr";
 
       if (priv->pseudo_class)
         pseudo_class = g_strconcat(priv->pseudo_class, " ",
@@ -727,7 +695,7 @@ st_widget_key_press_event (ClutterActor    *actor,
       (event->keyval == CLUTTER_KEY_F10 &&
        (event->modifier_state & CLUTTER_SHIFT_MASK)))
     {
-      g_signal_emit (actor, signals[POPUP_MENU], 0);
+      st_widget_popup_menu (ST_WIDGET (actor));
       return TRUE;
     }
 
@@ -777,6 +745,9 @@ st_widget_get_paint_volume (ClutterActor *self, ClutterPaintVolume *volume)
         {
           const ClutterPaintVolume *child_volume;
 
+          if (!clutter_actor_is_visible (child))
+            continue;
+
           child_volume = clutter_actor_get_transformed_paint_volume (child, self);
           if (!child_volume)
             return FALSE;
@@ -798,7 +769,7 @@ st_widget_real_get_focus_chain (StWidget *widget)
        child != NULL;
        child = clutter_actor_get_next_sibling (child))
     {
-      if (CLUTTER_ACTOR_IS_VISIBLE (child))
+      if (clutter_actor_is_visible (child))
         focus_chain = g_list_prepend (focus_chain, child);
     }
 
@@ -895,20 +866,6 @@ st_widget_class_init (StWidgetClass *klass)
                                                         ST_TYPE_THEME,
                                                         ST_PARAM_READWRITE));
 
-  /**
-   * StWidget:stylable:
-   *
-   * Enable or disable styling of the widget
-   */
-  pspec = g_param_spec_boolean ("stylable",
-                                "Stylable",
-                                "Whether the table should be styled",
-                                TRUE,
-                                ST_PARAM_READWRITE);
-  g_object_class_install_property (gobject_class,
-                                   PROP_STYLABLE,
-                                   pspec);
-
   /**
    * StWidget:track-hover:
    *
@@ -1570,7 +1527,6 @@ st_widget_init (StWidget *actor)
   StWidgetPrivate *priv;
 
   actor->priv = priv = ST_WIDGET_GET_PRIVATE (actor);
-  priv->is_stylable = TRUE;
   priv->transition_animation = NULL;
   priv->local_state_set = atk_state_set_new ();
 
@@ -1602,6 +1558,8 @@ st_widget_recompute_style (StWidget    *widget,
       return;
     }
 
+  _st_theme_node_apply_margins (new_theme_node, CLUTTER_ACTOR (widget));
+
   if (!old_theme_node ||
       !st_theme_node_geometry_equal (old_theme_node, new_theme_node))
     clutter_actor_queue_relayout ((ClutterActor *) widget);
@@ -1742,6 +1700,8 @@ st_widget_set_track_hover (StWidget *widget,
 
       if (priv->track_hover)
         st_widget_sync_hover (widget);
+      else
+        st_widget_set_hover (widget, FALSE);
     }
 }
 
@@ -1881,6 +1841,18 @@ st_widget_get_can_focus (StWidget *widget)
 
   return widget->priv->can_focus;
 }
+/**
+ * st_widget_popup_menu:
+ * @self: A #StWidget
+ *
+ * Asks the widget to pop-up a context menu.
+ */
+void
+st_widget_popup_menu (StWidget *self)
+{
+  g_signal_emit (self, signals[POPUP_MENU], 0);
+}
+
 
 /* filter @children to contain only only actors that overlap @rbox
  * when moving in @direction. (Assuming no transformations.)
@@ -1934,6 +1906,8 @@ filter_by_position (GList            *children,
             continue;
           break;
 
+        case GTK_DIR_TAB_BACKWARD:
+        case GTK_DIR_TAB_FORWARD:
         default:
           g_return_val_if_reached (NULL);
         }
@@ -1967,7 +1941,7 @@ sort_by_position (gconstpointer  a,
   /* Determine the relationship, relative to motion in @direction, of
    * the center points of the two actors. Eg, for %GTK_DIR_UP, we
    * return a negative number if @actor_a's center is below @actor_b's
-   * center, and postive if vice versa, which will result in an
+   * center, and positive if vice versa, which will result in an
    * overall list sorted bottom-to-top.
    */
 
@@ -1992,6 +1966,9 @@ sort_by_position (gconstpointer  a,
     case GTK_DIR_RIGHT:
       cmp = ax - bx;
       break;
+
+    case GTK_DIR_TAB_BACKWARD:
+    case GTK_DIR_TAB_FORWARD:
     default:
       g_return_val_if_reached (0);
     }
@@ -2017,6 +1994,8 @@ sort_by_position (gconstpointer  a,
     case GTK_DIR_RIGHT:
       fmid = (int)(sort_data->box.y1 + sort_data->box.y2) / 2;
       return abs (ay - fmid) - abs (by - fmid);
+    case GTK_DIR_TAB_BACKWARD:
+    case GTK_DIR_TAB_FORWARD:
     default:
       g_return_val_if_reached (0);
     }
@@ -2124,6 +2103,8 @@ st_widget_real_navigate_focus (StWidget         *widget,
             case GTK_DIR_RIGHT:
               sort_data.box.x2 = sort_data.box.x1;
               break;
+            case GTK_DIR_TAB_BACKWARD:
+            case GTK_DIR_TAB_FORWARD:
             default:
               g_warn_if_reached ();
             }
@@ -2317,7 +2298,7 @@ st_set_slow_down_factor (gfloat factor)
  * Returns: the global factor applied to all animation durations
  */
 gfloat
-st_get_slow_down_factor ()
+st_get_slow_down_factor (void)
 {
   return st_slow_down_factor;
 }
@@ -2536,8 +2517,6 @@ st_widget_remove_accessible_state (StWidget *widget,
 
 /* GObject */
 
-static void st_widget_accessible_class_init (StWidgetAccessibleClass *klass);
-static void st_widget_accessible_init       (StWidgetAccessible *widget);
 static void st_widget_accessible_dispose    (GObject *gobject);
 
 /* AtkObject */
@@ -2956,7 +2935,7 @@ check_labels (StWidgetAccessible *widget_accessible,
  *
  * Gets a list of the focusable children of @widget, in "Tab"
  * order. By default, this returns all visible
- * (as in CLUTTER_ACTOR_IS_VISIBLE()) children of @widget.
+ * (as in clutter_actor_is_visible()) children of @widget.
  *
  * Returns: (element-type Clutter.Actor) (transfer container):
  *   @widget's focusable children
